#!/usr/bin/python

from glob import glob
import os, sys, struct
from sys import argv
from stat import *

DGBMAXID = 0x4d424744

def readfile(name):
	try:
		fd = open(name, "rb")
		contents = fd.read()
		fd.close()
	except IOError:
		print "Error reading", name
		sys.exit(1)
	return contents

def writefile(name, contents):
	try:
		fd = open(name, "wb")
		fd.write(contents)
		fd.close()
	except IOError:
		print "Error writing", name
		sys.exit(1)

def col(color):
	c = int(color, 10)*8
	if c != 0:
		c += 7
	if c > 255:
		c = 255
	return c

def rgb(color):
	return struct.pack("BBB", col(color[0:2]), col(color[2:4]), col(color[4:6]))

def parsedat(file):
	txt = readfile(file)
	crcdict = {}
	line = 0
	region = ""
	name = ""
	for f in txt.splitlines():
		if line%3 == 0:
			region = f
		elif line%3 == 1:
			name = f
		else:
			crc = int(f[7:11], 16)
			crcdict[crc] = [ region, name ]
		line += 1
	return crcdict

def colors(col):
	cols = ""
	for i in xrange(0,12,3):
		r,g,b = struct.unpack("BBB", col[i:i+3])
		cols += "#%02d%02d%02d " % (r/8, g/8, b/8)
	return cols[0:-1]

def parse(file, out):
	dat = parsedat("dgbmax.dat")
	bin = readfile(file)
	txt = []
	if len(bin) & 3 != 0 or len(bin) < 44 or bin[0:4] != "DGBM":
		print "Invalid bin file"
		sys.exit(1)
	count = struct.unpack("<H", bin[4:6])[0]
	size = count * 38 + 6
	if (size&3):
		size += 2
	if len(bin) != size:
		print "Invalid bin file"
		sys.exit(1)
	for i in xrange(6,count*38+6,38):
		crc = struct.unpack("<H", bin[i:i+2])[0]
		if crc not in dat:
			print "CRC %04x not found" % crc
			sys.exit(2)
		region = dat[crc][0]
		name = dat[crc][1]
		if len(region) >= 69:
			txt.append("- %s -" % region)
		else:
			s = 73/2 - len(region)/2 - 1
			s2 = 73 - len(region) - 2 - s
			txt.append("%s %s %s" % ( "-" * s , region , "-" * s2))
		#print "-------------------------- DGBMax TXT Created ----------------------------"
		txt.append(name)
		txt.append("CRC: 0x%04X" % crc)
		j = i+2
		bg = colors(bin[j:j+12])
		j += 12
		win = colors(bin[j:j+12])
		j += 12
		# cut off last color
		c = colors(bin[j:j+12])[0:23]
		txt.append("BGPalette: %s" % bg)
		txt.append("OBJPalette: %s" % obj)
		txt.append("WINPalette: %s" % win)
	alltxt = "\r\n".join(txt)
	writefile(out, alltxt)

if __name__ == "__main__":
	if len(argv) != 3:
		print "Usage: %s dgbmax.bin dgbmax.ini"
		sys.exit(0)
	parse(argv[1], argv[2])
